# -----------------------------------------------------------------------------
package IDM_net_conns;
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA);
use IDM_Sigs;
use IDM_Template;
use Lsof;
use Netstat;

@ISA = ("IDM_Template", "IDM_Sigs");


# -----------------------------------------------------------------------------
# -- the eventual plan is to get network connection data via a bunch of 
#    different methods and compare the results:
#
#     -- implemented:
#          lsof
#          netstat
#
#     -- planned:
#            /proc/net/tcp 
#                      udp
#            api
#
# -----------------------------------------------------------------------------


# -- What?!  Global variables!

my @netstat_sigs_net_local = ();
my @netstat_sigs_net_pform = ();

my @lsof_sigs_net_local = ();
my @lsof_sigs_net_pform = ();


# -----------------------------------------------------------------------------

sub new() {
    my $class    = shift;
    my $config   = shift;    
    my $modsconf = shift;    
    my $self     = { };

    bless($self, $class);

    $self->new_template(CLASS      => $class,
                        SIDSCONFIG => $config,
                        MODSCONF   => $modsconf);

    $self->{CONN_S} = { };

    # -- grab sigs for expected network connections :
    #
    $self->_get_sigs_net("IDM_net_conns", $config->platform() . ".NETSTAT", \@netstat_sigs_net_pform);
    $self->_get_sigs_net("IDM_net_conns", "LOCAL.NETSTAT", \@netstat_sigs_net_local);
    #
    $self->_get_sigs_net("IDM_net_conns", $config->platform() . ".LSOF", \@lsof_sigs_net_pform);
    $self->_get_sigs_net("IDM_net_conns", "LOCAL.LSOF", \@lsof_sigs_net_local);

    return $self;
  }


# -----------------------------------------------------------------------------
# -- RUN :
# -----------------------------------------------------------------------------

# -- do our thing periodically :

sub run() {
    my $self = shift;
    my $sids = shift;

    my $modsconf = $self->{MODSCONF};

    $self->run_template($sids);

    if ($modsconf->mss(CLASS => $self->{CLASS}, CFG => "run_netstat_binary")) {
        $self->run_netstat_binary();
      }

    if ($modsconf->mss(CLASS => $self->{CLASS}, CFG => "run_lsof_binary")) {
        $self->run_lsof_binary();
      }

       # .../proc...
       # .../proc...

       # ...api...
       # ...api...

    if (    ($modsconf->mss(CLASS => $self->{CLASS}, CFG => "run_netstat_binary"))
         && ($modsconf->mss(CLASS => $self->{CLASS}, CFG => "run_lsof_binary"))) {

        # ...compare output ???...
        # ...compare output ???...
      }
  }

# -----------------------------------------------------------------------------


sub run_lsof_binary() {
    my $self  = shift;
    my $lsof  = Lsof->new(FLAGS => "-i4TCP", LOG => $self->{LOG});
    my $procs = $lsof->procs();

    my @bad_stuff = ();
    my @other     = ();

    foreach my $p (keys %$procs) {

        $self->debug_verbose("\n\n IDM_net_conns::run_lsof_binary : PROC  : "
                           . "\n\n $p ::"
                           . "\n\n IDM_net_conns::run_lsof_binary : FILES : ");

        foreach my $f (keys %{$lsof->open_files(PROC => $p)}) {

            my $c = {};

            $c->{COMMAND} = $lsof->proc_attrib(PROC=>$p, ATTRIB=>"COMMAND");
            $c->{UNAME}   = $lsof->proc_attrib(PROC=>$p, ATTRIB=>"UNAME");

            $c->{LOCAL_HOST}  = $lsof->file_attrib(PROC=>$p, FILE=>$f, ATTRIB=>"LOCAL_HOST");
            $c->{LOCAL_PORT}  = $lsof->file_attrib(PROC=>$p, FILE=>$f, ATTRIB=>"LOCAL_PORT");
            $c->{REMOTE_HOST} = $lsof->file_attrib(PROC=>$p, FILE=>$f, ATTRIB=>"REMOTE_HOST");
            $c->{REMOTE_PORT} = $lsof->file_attrib(PROC=>$p, FILE=>$f, ATTRIB=>"REMOTE_PORT");
            $c->{STATE}       = $lsof->file_attrib(PROC=>$p, FILE=>$f, ATTRIB=>"STATE");

            $self->debug_verbose("\n     " . $c->{LOCAL_HOST}  . ":" . $c->{LOCAL_PORT} 
                                    . "  " . $c->{REMOTE_HOST} . ":" . $c->{REMOTE_PORT}); 

            my ($flag_l, undef) = $self->_matches_net_sigs($c, \@lsof_sigs_net_local);
            my ($flag_p, undef) = $self->_matches_net_sigs($c, \@lsof_sigs_net_pform);
            
            # -- local takes precedence over platform --- this allows settings
            #    in expected.local.netstat to override those in 
            #    expected.<platform>.netstat :
            #
            if    ($flag_l eq "expected")  {                         }
            elsif ($flag_l eq "bad_stuff") {  push(@bad_stuff, $c);  }
	    elsif ($flag_p eq "expected")  {                         }
	    elsif ($flag_p eq "bad_stuff") {  push(@bad_stuff, $c);  }
            else                           {  push(@other, $c);      }
          }
      }


    #
    # -- log results : 
    #

    my $log = $self->{LOG};

    $log->output_title("LSOF BINARY");

    $log->output_array("Bad stuff", \@bad_stuff, 
                           ['%-16s', '$c->{COMMAND}', 
                            '%-16s', '$c->{UNAME}',
                            '%-32s', '  ::  $c->{LOCAL_HOST}:$c->{LOCAL_PORT}',
                            '%-8s',  '  <-->  ',
                            '%-32s', '$c->{REMOTE_HOST}:$c->{REMOTE_PORT}']);

    $log->output_array("Other stuff", \@other, 
                           ['%-16s', '$c->{COMMAND}', 
                            '%-16s', '$c->{UNAME}',
                            '%-32s', '  ::  $c->{LOCAL_HOST}:$c->{LOCAL_PORT}',
                            '%-8s',  '  <-->  ',
                            '%-32s', '$c->{REMOTE_HOST}:$c->{REMOTE_PORT}',
                            '%-32s' , 'STATE: $c->{STATE}']);
  }


# -----------------------------------------------------------------------------

sub run_netstat_binary() {

    my $self = shift;

    # -- the CPAN Net::Netstat Wrapper module does not work on Solaris 
    #    so we build our own :
    #
    my $modsconf = $self->{MODSCONF};
### my $netstat  = Netstat->new(PROTO => "TCPUDP", PFORM => $self->{SIDSCONFIG}->platform());
    my $netstat  = Netstat->new(PFORM => $self->{SIDSCONFIG}->platform());

    #
    # -- loop through connections and match, count... :
    #

    my $expected  = {};
    my @bad_stuff = ();
    my @other     = ();

    $self->debug_verbose( "\n\n IDM_net_conns::run_netstat_binary : CONNECTIONS : \n");

    foreach my $c (@{$netstat->connections()}) {

        $self->debug_verbose("\n $c->{LOCAL_HOST}:$c->{LOCAL_PORT} \t " 
                           . "$c->{REMOTE_HOST}:$c->{REMOTE_PORT} \t $c->{STATE}");

        my ($flag_l, $match_l) = $self->_matches_net_sigs($c, \@netstat_sigs_net_local);
        my ($flag_p, $match_p) = $self->_matches_net_sigs($c, \@netstat_sigs_net_pform);

        # -- local takes precedence over platform --- this allows settings
        #    in expected.local.netstat to override those in 
        #    expected.<platform>.netstat :
        #
        if    ($flag_l eq "expected")  {  $expected->{$match_l->{COUNTER}}++;  }
        elsif ($flag_l eq "bad_stuff") {  push(@bad_stuff, $c);                }
	elsif ($flag_p eq "expected")  {  $expected->{$match_p->{COUNTER}}++;  }
	elsif ($flag_p eq "bad_stuff") {  push(@bad_stuff, $c);                }
        else                           {  push(@other, $c);                    }
      }

    #
    # -- log results : 
    #

    my $log = $self->{LOG};

    $log->output_title("NETSTAT BINARY");
    $log->output_hash("Expected", $expected, ['%-16s', '%4s']); 

    $log->output_array("Bad stuff", \@bad_stuff, 
                           ['%-32s', '  ::  $c->{LOCAL_HOST}:$c->{LOCAL_PORT}',
                            '%-8s',  '  <-->  ',
                            '%-32s', '$c->{REMOTE_HOST}:$c->{REMOTE_PORT}',
                            '%-32s' , 'STATE: $c->{STATE}']);

    $log->output_array("Other stuff", \@other, 
                           ['%-32s', '  ::  $c->{LOCAL_HOST}:$c->{LOCAL_PORT}',
                            '%-8s',  '  <-->  ',
                            '%-32s', '$c->{REMOTE_HOST}:$c->{REMOTE_PORT}',
                            '%-32s' , 'STATE: $c->{STATE}']);
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
