# -----------------------------------------------------------------------------
package IDM_procs;
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA);
use IDM_Sigs;
use IDM_Template;
use Lsof;
use Proc::ProcessTable;

@ISA = ("IDM_Template", "IDM_Sigs");


# -----------------------------------------------------------------------------
# -- sub-modules :
#
#     -- /proc/
#     -- Proc::ProcessTable (i.e., api)
#     -- ps wrapper
#
# -----------------------------------------------------------------------------

my $ptable_sigs_pform = {};
my $ptable_sigs_local = {};

my $lsof_sigs = {};

# -----------------------------------------------------------------------------

sub new() {
    my $class    = shift;
    my $config   = shift;    
    my $modsconf = shift;    
    my $self     = { };

    bless($self, $class);

    $self->new_template(CLASS      => $class,
                        SIDSCONFIG => $config,
                        MODSCONF   => $modsconf);

    $self->_get_procs_api_sigs("LOCAL.API", $ptable_sigs_local);
    $self->_get_procs_api_sigs($config->platform() . ".API", $ptable_sigs_pform);
                                  # ...TODO: consider possibility of having
                                  # ...TODO: local override platform --- but what
                                  # ...TODO: if have system sshd and local sshd?


    $self->_get_procs_lsof_sigs($config->platform . ".LSOF", $lsof_sigs);
    $self->_get_procs_lsof_sigs("LOCAL.LSOF", $lsof_sigs);
        # ...for lsof we have only one set of sigs with local overriding
        #    platform...

    return $self;
  }

# -----------------------------------------------------------------------------
# --

sub run () {
    my $self = shift;
    my $sids = shift;

    my $modsconf = $self->{MODSCONF};

    $self->run_template($sids);

    if ($modsconf->mss(CLASS => $self->{CLASS}, CFG => "run_api")) {
        $self->_run_api();
      }

    if ($modsconf->mss(CLASS => $self->{CLASS}, CFG => "run_lsof")) {
        $self->_run_lsof();
      }
  }


# -----------------------------------------------------------------------------
# --

sub _run_lsof() {
    my $self = shift;
    my $log  = $self->{LOG};

    $log->append("");
    $log->append("------------------------------------------------------------------------------");
    $log->append_echo("Getting process details via lsof...");
    my $lsof  = Lsof->new(FLAGS => "+M", LOG => $self->{LOG});
    my $procs = $lsof->procs();
    $log->append_echo("     ...got...");
    $log->append("------------------------------------------------------------------------------");


    $log->append("");
    $log->append("------------------------------------------------------------------------------");
    $log->append("-- LSOF :: Checking root process sigs ----------------------------------------");
    $log->append("");
    $log->indent();

    foreach my $p (keys %$procs) {

##      $procs->{$p}->{PS}->{COMMAND} =~ s/\///g;
        my $cmd  = $lsof->proc_attrib(PROC=>$p, ATTRIB=>"COMMAND");
        $cmd =~ s/\///g;
        $lsof->reset_proc_attrib(PROC=>$p, ATTRIB=>"COMMAND", VALUE=>$cmd);
            # ...smp linux reports e.g., "kprocess/0" or "kthing/1" which upsets us...

        unless ($lsof->proc_attrib(PROC=>$p, ATTRIB=>"UNAME") eq "root") { next; }

##      unless ($self->__matches_expected_root_procs_sigs($lsof, $p, $procs->{$p})) {
        unless ($self->__matches_expected_root_procs_sigs($lsof, $p)) {
	    $log->append("Does not match a sig :  " 
                       . sprintf("%10s", $lsof->proc_attrib(PROC=>$p, ATTRIB=>"PPID"))  
                       . sprintf("%10s", $lsof->proc_attrib(PROC=>$p, ATTRIB=>"GID"))  
                       . "    " . $lsof->proc_attrib(PROC=>$p, ATTRIB=>"COMMAND"))
          }
      }
    $log->un_indent();
    $log->append("");
    $log->append("...checking root process signatures.");
    $log->append("");
  }


# -----------------------------------------------------------------------------
# --

sub _run_api() {
    my $self = shift;
    my $log  = $self->{LOG};

    $log->append("");
    $log->append("------------------------------------------------------------------------------");
    $log->append_echo("Getting process table (via Proc)...");
    my $PPT   = Proc::ProcessTable->new();
    my $table = $PPT->table();
    $log->append_echo("     ...got...");
    $log->append("------------------------------------------------------------------------------");

    $log->append("");        
    $log->append("------------------------------------------------------------------------------");
    $log->append("-- ProcessTable :: -----------------------------------------------------------");
    $log->append("");

    $log->append("Unexpected at :");        
    $log->append("");
    $log->indent();
    $log->append(sprintf("%10s", "PID") . "  " . 
                 sprintf("%10s", "UID") . "  " .  
                 sprintf("%10s", "USERNAME") . "  " .  
                 sprintf("%-15s", "FILENAME") . "  " .  
                 "CMNDLINE");
    $log->append(""); 
        
    foreach my $p (@$table) {
        my $matches_l = $self->_matches_proc_sigs($p, $ptable_sigs_local);
        my $matches_p = $self->_matches_proc_sigs($p, $ptable_sigs_pform);
          
        unless ($matches_l || $matches_p) {
            my $str = $self->__proc_unique_str($p);
            $log->append($str);        
          }
      }
    $log->un_indent();
  }


# -----------------------------------------------------------------------------
# -- _run_api
# -----------------------------------------------------------------------------

# -- generate a unique string with which to identify each process :

sub __proc_unique_str() {
    my $self = shift;
    my $proc = shift;

    # ...catenate PDI, UID of owner, username of owner, filename of exe
    #    that is running and command line with which it started...

    return sprintf("%10s", $proc->pid())
           . "  " . sprintf("%10s", $proc->uid())
           . "  " . sprintf("%10s", getpwuid($proc->uid()))
           . "  " . sprintf("%-15s", $proc->fname())
           . "  " . $proc->cmndline();
  }


# -----------------------------------------------------------------------------
# -- _run_lsof :
# -----------------------------------------------------------------------------

# -- :

sub __matches_expected_root_procs_sigs() {
    my $self = shift;
    my $lsof = shift;
    my $pk   = shift;
##    my $ps   = shift;
    my $cmd  = $lsof->proc_attrib(PROC=>$pk, ATTRIB=>"COMMAND");
    my $log  = $self->{LOG};

    # -- default is no match :
    #
    my $result  = 0;

##  my @keys    = keys %$lsof_sigs;
    my $details = { };
  
    $details->{CWD}  = [ ];
    $details->{AOUT} = [ ];
    $details->{ODS}  = [ ];
    $details->{OFS}  = [ ];

    # -- if $lsof_sigs have a key of which the end matches $cmd :
    #
##  foreach my $k (@keys) {
    foreach my $k (keys %$lsof_sigs) {
        my $tmp = $k;    $tmp =~ s/^.*\///;
            # .../abc/def/cmd --> cmd...

        if (    ($cmd eq $tmp)
             && ($self->_matches_procs_lsof_aout($lsof, $pk, $lsof_sigs->{$k}, $details->{AOUT}))) {
            if (    $self->_matches_procs_lsof_cwd($lsof, $pk, $lsof_sigs->{$k}, $details->{CWD})
                 && $self->_matches_procs_lsof_ods($lsof, $pk, $lsof_sigs->{$k}, $details->{ODS})
                 && $self->_matches_procs_lsof_ofs($lsof, $pk, $lsof_sigs->{$k}, $details->{OFS})
#                && $self->__open_inet_conns_sig_ok($ps, $lsof_sigs)
                                             # 
                                             # ...TODO open inet conns...
                                             # ...TODO open inet conns...
               ) {
                $result = 1;
              }
          }
     } 

    foreach my $k (keys %$details) {
        if (($result == 0) && (scalar @{$details->{$k}} >= 1)) {
            $log->indent();
            $log->append("");
            foreach (@{$details->{$k}}) {  $log->append($_);  }
            $log->un_indent();
          }
      }

    return $result;
  }


# -----------------------------------------------------------------------------
# --

sub __open_inet_conns_sig_ok($ps) {
    my $self   = shift;
    my $ps     = shift;
    my $sigs   = shift;
    my $cmd    = $ps->{PS}->{COMMAND};
    my $log    = $self->{LOG};
    my $result = 0;

    if (defined $sigs->{$cmd}->{INET}) {

        # -- compute the symmetric difference between the expected list
        #    of open devices and the list found:

        my %count = ();

        foreach my $e (keys %{$ps->{FILES}}) {
            unless ($ps->{FILES}->{$e}->{TYPE} eq "inet") {next;}
                # ...connections only...

            $count{$ps->{FILES}->{$e}->{NAME}}++;
          }

        # -- increment count of expected such files :
        foreach my $e (@{$sigs->{$cmd}->{INET}}) {$count{$e}++;}

        my @diff = ();
        foreach my $e (keys %count) {
            # -- if found not once then we have a difference :
            if ($count{$e} == 1) { push(@diff, $e) }
          }

        # -- log any found differences :
        $log->indent();
        foreach (@diff) {$log->append("open inet sig diff: $_");}
        $log->un_indent();
	if (scalar @diff > 0) {
            $log->append("Open-inets signature does not match: $cmd"); 
	    $result = 0;
          }
        else {  $result = 1;  }
      }
    else {  $result = 1;  }

    return $result;
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
